/* FILE TALD3E_SM_EXPORT.C; exportized version of:       */
/*********************************************************/
/* tald3e_sm.c - tests recursive alignment distance function*/
/* by creating file of phonological alignments based on  */
/* contents of an input alignment file.                  */
/* This version does split/merge alignment.              */
/*                                                       */
/* Synopsis: tald3e_sm <parms>, where:                   */
char *parms = "fname_aln_out fname_aln_in fname_pcdtab fname_ttp_pcdrls sylb_data_dir fom_threshold lim (db_level)";
/*                                                       */
/* Input:                                                */
/*   the old alignments from file "fname_aln_in"         */
/* Output:                                               */
/*   new alignments into file "fname_aln_out             */
/* Parameters:                                           */
/*   fname_pcdtab: name of file containing a tabular     */
/*     definition of the particular phonological code    */
/*     sets (pcodesets) to be used.                      */
/*   fname_ttp_pcdrls: name of file containing TTP rules */
/*     for making new lexicon entries, as a pcodeset.    */
/*   sylb_data_dir: directory where syllabification data */
/*     can be found.                                     */
/*   fom_threshold: if split/merge figure of merit (fom) */
/*     is greater than this value, s/m hypothesis is     */
/*     accepted.  Typical value would be 1.50.           */
/*   lim: arbitrary limit to the number of alignments    */
/*     that will be processed (for testing).             */
/*   db_level: level of printout of intermediate results */
/*     (de-bug print-outs); 0 for none, 1 for a few, etc.*/
/*                                                       */
/* Author: W. M. Fisher, NIST                            */
/* Release history:                                      */
/*   DATE  VER DESCRIPTION                               */
/*  6/8/98 1.0 Major improvement to TTP function that is */
/*     used to make prons for words not in dictionary,   */
/*     fixed all memory leaks.                           */
/* 12/5/97 0.1 Includes fixes to one bug that would be   */
/*     triggered if two candidate s/m's are close enough */
/*     to conflict, and some memory allocation and       */
/*     initialization problems.                          */
/*10/15/97 0.0 Original release.                         */
/*********************************************************/

/* standard compile environment files: */
/* #include <gp/stdcenvp.h> */ /* NIST local version */
#include "stdcenvp.h" /* export version */


int main(int argc, char *argv[])
 {char *proc = "tald3e_sm";
  char *ver_no = "Ver. 1.0";
  TABLE_OF_CODESETS codesets, *pcdtab = &(codesets);
  pcodeset *pcode_x, *pcode_word, *pcode_phon;
  pcodeset ttp_pcdrls_data, *ttp_pcdrls = &ttp_pcdrls_data;
  struct BPATH ttp_bpath_data, *ttp_bpath = &ttp_bpath_data;
  struct SYL_PARMS ttp_syl_parm_data, *ttp_syl_parms = &ttp_syl_parm_data;
  TTP_PCDRLS_PARAMETERS ttparmx, *ttp_pcdrls_parms = &ttparmx;
  char sxx[LONG_LINE], *sx = &sxx[0];
  char *fname_pcdtab, *fname_ttp_pcdrls, *fname_aln_in, *fname_aln_out;
  char *sylb_data_dir;
  char *path, ttp_tgbound = '.';
  ALIGNMENT_INT *p1,*p2,*p3;
  ALIGNMENT_CHR *ps1;
  boolean return_alignment = T, at_end, print_sms = T;
  int d, nal_total, lim, err,*perr = &err;
  int i, ichr, jchr;
  double fom_threshold;
  FILE *fp_in, *fp_out; /* corpus file */
  time_t t_sec;
/*   code   */
/* global initialization */
  t_sec = time(&t_sec);      /* ANSI, Turbo C */
  pdb = strcpy(pdb,"*DB: ");
  detail_level = 0;
  pcdtab->ncodesets = 0;
/* get command line parameters */
  if (argc < 8)
    {fprintf(stdout,"Usage: %s %s\n",proc,parms);
     return -1;
    }
  fname_aln_out     = argv[1];
  fname_aln_in      = argv[2];
  fname_pcdtab      = argv[3];
  fname_ttp_pcdrls  = argv[4];
  sylb_data_dir     = argv[5];
  fom_threshold     = atof(argv[6]);
  lim               = atoi(argv[7]);
  if (argc > 8) db_level = atoi(argv[8]); else db_level = 0;
/* report basics */
  fprintf(stdout," Program %s %s Report.\n",proc,ver_no);
  fprintf(stdout,"\n");
  fprintf(stdout," Produces new alignment file from old.\n");
  fprintf(stdout," Run %s.\n",del_eol(ctime(&t_sec)));
  fprintf(stdout,"\n");
  fprintf(stdout," Command line parameters:\n");
  fprintf(stdout,"   Old alignments file: '%s'\n",fname_aln_in);
  fprintf(stdout,"   New alignments file: '%s'\n",fname_aln_out);
  fprintf(stdout,"   pcode   table  file: '%s'\n",fname_pcdtab);
  fprintf(stdout,"   TTP   pcdrls   file: '%s'\n",fname_ttp_pcdrls);
  fprintf(stdout,"   sylb data directory: '%s'\n",sylb_data_dir);
  fprintf(stdout,"   fom threshold: %f\n",fom_threshold);
  fprintf(stdout,"   output limit: %d triples\n",lim);
  fprintf(stdout,"   db_level: %d\n",db_level);
/* get codesets */
  get_code_table(pcdtab, fname_pcdtab, &err);
  if (err > 10)
    {fprintf(stdout," *ERR:Get_code_table returned err code =%d\n",err);
     fprintf(stdout," file: '%s'\n",fname_pcdtab);
     return 1;
    }
if (db_level > 0) dump_code_table(pcdtab);
  if (pcdtab->ncodesets < 0)
    {fprintf(stdout,"; *ERR:Get_code_table returns ncodesets < 1\n");
     return 2;
    }
/* find word and phon pcodesets */
  pcode_word = NULL;
  pcode_phon = NULL;
  for (i = 1; i <= pcdtab->ncodesets; i++)
    {pcode_x = pcdtab->cd[i].pc;
     if (strstr(pcode_x->element_type,"word")) pcode_word = pcode_x;
     if (strstr(pcode_x->element_type,"phon")) pcode_phon = pcode_x;
    }
  if (pcode_word == NULL) fatal_error(proc,"No word-level pcodeset defined.",999);
  if (pcode_phon == NULL) fatal_error(proc,"No phoneme-level pcodeset defined.",999);

/* for s/m logic to work, pcode_word must be compositional */
  if (!pcode_word->compositional)
     fatal_error(proc,"Word-level pcodeset is not compositional.",999);
  else
    {ttp_pcdrls_parms->desc = strdup_safe("Standard TTP3S Parameters",proc);
/* get TTP parameters for making new words */
if (db_level > 0) printf("%s Reading in ttp pcdrls...\n",pdb);
     /* get ttp rules from ttp pcdrls file */
       make_pcd_from_rls_file(ttp_pcdrls,fname_ttp_pcdrls,path="",ttp_tgbound,perr);
       if (*perr > 0) fatal_error(proc,"make_pcd_from_rls_file fails",*perr);
       ttp_pcdrls_parms->pcdrls = ttp_pcdrls;
     /* get ttp back-off path */
       get_bpath_from_pcdrls1(ttp_bpath,ttp_pcdrls,ttp_tgbound,perr);
       if (*perr > 0) fatal_error(proc,"error return from get_bpath...",*perr);
if (db_level > 0) dump_bpaths(ttp_bpath);
       ttp_pcdrls_parms->bpath = ttp_bpath;
      /* get syllabification parameters */
       ttp_syl_parms->desc     = strdup_safe("Syllabification parameters.",proc);
       ttp_syl_parms->pcdp_dir = pcdtab->directory;
       ttp_syl_parms->data_dir = sylb_data_dir;
       ttp_syl_parms->pcdp     = pcode_phon;
if (db_level > 0) printf("%s Reading in syllabification parameters...\n",pdb);
       read_syl_parms(ttp_syl_parms,perr);
       if (*perr > 0) fatal_error(proc,"error return from read_in_syl_parms()",*perr);
       if (db_level > 0) dump_syl_parms(ttp_syl_parms);
       ttp_pcdrls_parms->syl_parms = ttp_syl_parms;
      }
if (db_level > 0) dump_syl_parms(ttp_syl_parms);

/* initialize score-keeping counters */
  nal_total = 0;
/* make alignment data objects */
  p1 = make_al_int(pcode_word);
  p2 = make_al_int(pcode_word);
  p3 = make_al_int(pcode_word);
  ps1 = make_al_chr();
 /* open alignment files */
  if (streq(fname_aln_in,"-")) fp_in = stdin;
  else
    {fp_in = fopen(fname_aln_in,"r");
     if (fp_in == NULL) fatal_error(proc,"Can't open fname_aln_in",999);
    }
  if (streq(fname_aln_out,"-")) fp_out = stdout;
  else
    {fp_out = fopen(fname_aln_out,"w");
     if (fp_out == NULL) fatal_error(proc,"Can't open fname_aln_out",999);
    }
if (db_level > 0) printf("%s Files opened.\n",pdb);
 /* and loop on contents */
if (db_level > 0) printf("%s Looping on contents.\n",pdb);
  at_end = F;
  while ((!at_end)&&(nal_total < lim))
    {get_next_alignment_chr(fp_in,ps1,&at_end); /* read in next alignment */
     if (!at_end)
       {simplify_null_reps(ps1->pastr);
        simplify_null_reps(ps1->pbstr);
        adjust_type_case(ps1->pastr,pcode_word);
        adjust_type_case(ps1->pbstr,pcode_word);
        if (ntokens(ps1->pastr) != ntokens(ps1->pbstr)) ps1->aligned = F;
        alignment_s_to_ir(p1,ps1,pcode_word,ttp_pcdrls_parms,perr);
        if (*perr > 0) fatal_error(proc,"alignment_s_to_ir fails",*perr);
        p2 = copy_int_al(p2,p1);
        remove_zeros(&(p2->aint[0]));
        remove_zeros(&(p2->bint[0]));
        p2->aligned = F;
        d=aldistsm(p2,wod2,return_alignment,fom_threshold,p3,perr); /* re-align it */
        if (*perr > 10) fatal_error(proc,"aldistsm fails",*perr);
        fprintf(fp_out,"\n");
        fprintf(fp_out,"id: (%s)\n",ps1->id);
        aln_i_to_s_fancy2(p3,ps1,perr);
        if (*perr > 0) fatal_error(proc,"aln_i_to_s_fancy2 fails",*perr);
        fprintf(fp_out,"REF: %s\n",ps1->pastr);
        fprintf(fp_out,"HYP: %s\n",ps1->pbstr);
        nal_total += 1;
      /* print out s/m's found if desired*/
        if ((print_sms)&&(p3->naux > 0))
          {if (nal_total == 1)
             {fprintf(stdout,"# SPLIT/MERGES FOUND:\n");
              fprintf(stdout,"#      CASE I.D.   REF::HYP\n");
             }
           for (i=1; i <= p3->aint[0]; i++)
             {ichr = p3->aint[i];
              jchr = p3->bint[i];
              if ((ichr > pcode_word->pctab_length)
                ||(jchr > pcode_word->pctab_length))
                {fprintf(stdout,"%16.16s  ",ps1->id);
                 fprintf(stdout," %s",ascii_str(sx,ichr,pcode_word,p3));
                 fprintf(stdout," :: ");
                 fprintf(stdout,"%s\n",ascii_str(sx,jchr,pcode_word,p3));
          }  }  }
	/* free auxtab in output alignment */
         for (i=1; i <= p1->naux; i++)
           {if (p1->auxtab[i] != NULL)
              {free(p1->auxtab[i]);
              }
           }
         p1->naux = 0;
     }  }
   fprintf(stdout,"\n");
   fprintf(stdout," No. of alignments processed=%d\n",nal_total);
 /* free allocated memory */
   free(ttp_pcdrls_parms->desc);
   free_pcode(ttp_pcdrls_parms->pcdrls);
   free_code_table(pcdtab);
   free_syl_parms2(ttp_syl_parms);
   free_bpath1(ttp_bpath);
   free_al_int(p1);
   free_al_int(p2);
   free_al_int(p3);
   free_al_chr(ps1);
   return 0;
  } /* end main */

/******* CODE FOR FUNCTIONS USED, FOR EXPORT VERSION: ************************/
/* (you may uncomment these lines to include the code directly)              */
#include "acstr.c"
#include "addpcd.c"
#include "adjcase.c"
#include "aldist2.c"
#include "aldistsm.c"
#include "aldistst.c"
#include "alitos2.c"
#include "alitos4.c"
#include "alstoi2r.c"
#include "alstoi3.c"
#include "aprules2.c"
#include "atobool.c"
#include "boolpr1.c"
#include "build_key3.c"
#include "calloc2.c"
#include "char_name.c"
#include "copintal.c"
#include "dauxtab.c"
#include "dbpaths.c"
#include "dbpkg1.c"
#include "dchral.c"
#include "dctable.c"
#include "decode2.c"
#include "del_eol.c"
#include "dhpars1.c"
#include "dintal2.c"
#include "dintal4.c"
#include "dpcode2.c"
#include "dpcode3.c"
#include "dppf2.c"
#include "dpron2.c"
#include "drules2.c"
#include "dsylparms.c"
#include "dwords1.c"
#include "dsylbs1.c"
#include "enclcs1.c"
#include "encode2.c"
#include "encode2r.c"
#include "expenv.c"
#include "fatalerr.c"
#include "fbpath1.c"
#include "find_key3.c"
#include "fntoke2.c"
#include "fralchr.c"
#include "fralint.c"
#include "frctable.c"
#include "frele.c"
#include "frpcode.c"
#include "frstr1.c"
#include "frules2.c"
#include "fsylparms2.c"
#include "ftiers2.c"
#include "gbpath1.c"
#include "gcomflag.c"
#include "gctable.c"
#include "gpcaux.c"
#include "gpcode3.c"
#include "gppf2.c"
#include "gnalc1.c"
#include "gnstrtab.c"
#include "grules2.c"
#include "hashfcn.c"
#include "initaln2.c"
#include "initalc2.c"
#include "ipcdrls.c"
#include "lc_istr.c"
#include "letfcns.c"
#include "make_pcd_from_rls_file.c"
#include "malchr.c"
#include "malint.c"
#include "malloc2.c"
#include "mbalint.c"
#include "mfname1.c"
#include "mkele1.c"
#include "mkele1r.c"
#include "mpron4.c"
#include "mlower.c"
#include "mupper.c"
#include "no_wbndl.c"
#include "no_wbndr.c"
#include "nstrfcns.c"
#include "ntokens.c"
#include "okfcc2.c"
#include "okicc2.c"
#include "overlap1.c"
#include "padwsp.c"
#include "parse_rkey.c"
#include "pchmeth1.c"
#include "pcindex2.c"
#include "pcindex2r.c"
#include "pcindex4.c"
#include "pcindex5.c"
#include "pcsort1.c"
#include "pltrim.c"
#include "pltrimf.c"
#include "pronphon.c"
#include "prtrim.c"
#include "prtrim2.c"
#include "rdiacrit.c"
#include "remcomm.c"
#include "remzeros.c"
#include "repladd2.c"
#include "rsylparms.c"
#include "safepr1.c"
#include "simpnull.c"
#include "sm_pkg1.c"
#include "spmode.c"
#include "ss_len.c"
#include "ss_to_s.c"
#include "sstok.c"
#include "sstok2.c"
#include "sstok4.c"
#include "str_eq.c"
#include "sstok_lr_s.c"
#include "strcmpi.c"
#include "strcutl.c"
#include "strcutr.c"
#include "strdup2.c"
#include "strmap1.c"
#include "strncmpi.c"
#include "strseg.c"
#include "sylpreds.c"
#include "textlen.c"
#include "ttp_prep1.c"
#include "ttp1.c"
#include "ttp1s.c"
#include "ttp3s.c"
#include "ttpj1.c"
#include "ttpspan2.c"
#include "ttp_pcdrls2ss.c"
#include "valdata.c"
#include "warn_add.c"
#include "wbtrim.c"
#include "wod2.c"
#include "zapchar.c"
